// LCD-TEST.C - Copyright 1998, Technologic Systems, Inc.
// 	permission is granted for you to use this source code freely.
//
//	This program is a simple example for use with the Technologic
//	Systems line of Embedded PCs. If the platform is a TS2350 with
//	an LCD installed, LCD-TEST will display the time and date on the
//	LCD using the standard printf function. If an LCD is not present, 
//	the program will use the regular display, or the serial console.
//
//	Press the board-mounted push-switch to exit.
//
//	This program will run without modification on a standard
//	desktop PC.
//
//	Note that two thirds of the code in this program is used to
//	simply determine what platform the program is running on, then
//	turn the LCD and serial redirection on or off. The actual main
//	loop of code is only 9 lines long.
//
// cjd  980910  1.0     Birth
// cjd	990323	1.1	Added '\n' to mesg to display properly on 2x40 LCDs
// cjd	990904	1.2	Fixed to recognize all TS EPC models - not just 2350
//


#include   <stdio.h>
#include   <dos.h>
#include   <stdlib.h>
#include   <conio.h>

const char VER[] = "LCD-TEST v1.2, (c) 1999 Technologic Systems Inc.";

// Check whether we are running on a TS EPC
int CheckTSBios ()
{
	union REGS regs;

	regs.x.ax = 0xB000;		// TS BIOS info function
	int86 (0x15, &regs, &regs);     // do it.

	if (regs.h.ah == 0 && regs.x.dx == 0x5453) // 'TS' should be returned in DX
		return (regs.h.cl);	// YES - return the model number
	else
		return (0);		// NO this is NOT TS BIOS
} // CheckTSBios

// Make the BIOS call to change the console port. 0 = Video, 1 = COM1, etc
void SetConsolePort (int port)
{
	union REGS regs;

	regs.h.ah = 0xA1;		// Set console redirection function
	regs.x.bx = port;		// Set which port
	int86 (0x15, &regs, &regs);     // do it.
}

// Return the current state of the push switch.
int GetPswStatus ()
{
	union REGS regs;

//	if (!modelnum)			// Non-TS EPC platform
//		return 0;

	regs.x.ax = 0xB021;		// PSW Status function
	int86 (0x15, &regs, &regs);     // do it.
	if (!regs.h.ah && regs.x.bx)	// PSW pressed?
		return 1;		// .. yes
	else
		return 0;		// .. no
}

int main(void)
{
	struct	time t;
	struct	date d;

	union	REGS regs;

	int	modelnum = 0,
		redirect = 0;

	printf ("%s\n", VER);		// print program version

	// If this is a TS2350 and an LCD display is hooked up, display
	// the time and date on the LCD
	printf ("Checking platform: ");
	modelnum = CheckTSBios ();
	if (modelnum == 0)
		printf ("Non-Technologic Computer.\n");
	else
		printf ("Technologic Systems EPC Model ");
		switch (modelnum) {
			case 13:
				printf ("TS1325.\n");
				break;
			case 21:
				printf ("TS2100.\n");
				break;
			case 22:
				printf ("TS2200.\n");
				break;
			case 23:
				printf ("TS2325.\n");
				break;
			case 24:
				printf ("TS2350.\n");
				break;
			case 28:
				printf ("TS2800.\n");
				break;
			case 45:
				printf ("TS4500.\n");
				break;
			default:
				printf ("%d.\n", modelnum);
		}
	
	// some EPC models have an LCD interface, some don't.
	// If this is a TS EPC, we will go ahead and try to init the LCD
	// If the init fails, we can check the return value for whether the failure
	// was caused by no LCD interface at all, or the LCD just failed to init.
	if (modelnum != 0) {		// is this a TS EPC?
		printf ("Attempting LCD enable... ");
		regs.x.ax = 0xB042;	// yes -- attempt LCD enable
		regs.h.bh = 0x01;
		int86 (0x15, &regs, &regs);

		if (regs.h.ah == 0 && regs.h.bh == 1) {
			printf ("Success!\n");
			SetConsolePort (0);  // console redirection must be off for LCD display	
			redirect = 1;	// flag for redirection
		} else {
			redirect = 0;
			printf ("Failed - ");
			switch (regs.h.ah) {
				case 1:
					printf ("no LCD interface present.\n");
					break;
				case 3:
					printf ("is LCD plugged in?\n");
					break;
				default:
					printf ("** Unknown error **\n");
			}
		}
	}

	// MAIN PROGRAM LOOP
	while (!GetPswStatus() && !kbhit()) {
		gettime (&t);
		getdate (&d);

		system ("cls");

		printf ("Hit key or PSW to exit\n");
		printf ("  %2d:%02d:%02d  %d-%02d-%02d ",
			t.ti_hour, t.ti_min, t.ti_sec,
			d.da_year, d.da_mon, d.da_day);

		sleep (1);
	}

	// If an LCD interface is enabled, disable it now
	if (redirect) {
		printf ("Attempting LCD disable... ");
		system ("cls");
		regs.x.ax = 0xB042;	// turn off LCD
		regs.h.bh = 0x00;
		int86 (0x15, &regs, &regs);

		if (regs.h.ah == 0 && regs.h.bh == 0) {
			printf ("Success!\n");
			SetConsolePort (2);
		} else
			printf ("Failed.\n");
	}

	printf ("\n");

	return 0;
}
